#version 330 core

in vec4 WS_pos_from_GS; 
in vec3 f_worldNormal;
in vec3 f_materialColor;
in vec2 f_tex_uv;
in vec3 f_directionToLight;
in vec3 f_directionToCamera;

uniform sampler2D colorTexture;

out vec4 fragColor;       //final fragment color

//calculate diffuse component of lightning
float diffuseComponent(vec3 L, vec3 N){
	return clamp(dot(L, N), 0.0, 1.0);
} 

//calculate specular component of lightning
float specularComponent(vec3 L, vec3 N, vec3 H){
	if(dot(N, L) > 0){
		return pow(clamp(dot(N, H), 0.0, 1.0), 32.0);
	}
	return 0.0;
}

void main()
{
	//sample color from diffuse texture
	vec3 color = texture2D(colorTexture, f_tex_uv).rgb;
	//calculate total intensity of lightning
	vec3 halfVector = normalize(f_directionToLight + f_directionToCamera);
	float ambientIntensity = 0.3;
	float diffuseIntensity = diffuseComponent(f_directionToLight, f_worldNormal);
	float specularIntensity = specularComponent(f_directionToLight, f_worldNormal, halfVector);

	float totalIntensity = ambientIntensity + 0.7 * diffuseIntensity + specularIntensity;

	//use color from texture
	fragColor.xyz = color * totalIntensity; 
	fragColor.w = 1.0;


	// calculate distance
//	float WS_dist = distance(WS_pos_from_GS, l_pos);
	// map value to [0;1] by dividing by far plane distance
//	float WS_dist_normalized = WS_dist geteilt durch near_far.y;
	// write modified depth
//	gl_FragDepth = WS_dist_normalized;
	// when using depth-only FBO, do NOT write to color!!!
}



