#version 330 core

in vec4 WS_pos_from_GS; 
in vec3 f_worldNormal;
in vec3 f_materialColor;
in vec2 f_tex_uv;
in vec3 f_directionToLight;
in vec3 f_directionToCamera;
in vec4 f_Shadow_tex_coord;

uniform sampler2DShadow shadowMap;  //shadow map is just a texture
uniform sampler2D colorTexture;
//uniform vec2 near_far;  near and far plane for cm-cams
//uniform vec4 l_pos;  world space light position

out vec4 fragColor;       //final fragment color

//calculate diffuse component of lightning
float diffuseComponent(vec3 L, vec3 N){
	return clamp(dot(L, N), 0.0, 1.0);
} 

//calculate specular component of lightning
float specularComponent(vec3 L, vec3 N, vec3 H){
	if(dot(N, L) > 0){
		return pow(clamp(dot(N, H), 0.0, 1.0), 32.0);
	}
	return 0.0;
}

void main()
{
	//sample color from diffuse texture
	vec3 color = texture2D(colorTexture, f_tex_uv).rgb;
	//calculate total intensity of lightning
	vec3 halfVector = normalize(f_directionToLight + f_directionToCamera);
	float ambientIntensity = 0.3;
	float diffuseIntensity = diffuseComponent(f_directionToLight, f_worldNormal);
	float specularIntensity = specularComponent(f_directionToLight, f_worldNormal, halfVector);

	//perform perspective division - homogenisierung
	vec3 tex_coords = f_Shadow_tex_coord.xyz/f_Shadow_tex_coord.w;

	//read depth value from shadow map
	float inShadow = texture(shadowMap, tex_coords.xyz - vec3(0,0,0.004));

	float totalIntensity = ambientIntensity + 0.7 * (diffuseIntensity * inShadow) + (specularIntensity * inShadow);

	//use color from texture
	fragColor.xyz = color * totalIntensity; 
	fragColor.w = 1.0;
}



