#version 330 core

in vec4 WS_pos;
in vec4 WS_camPos;
in vec4 Shadow_tex_coord;         
in vec3 worldNormal;
in vec3 materialColor;
in vec2 tex_uv;
in vec3 directionToLight;
in vec3 directionToCamera;

uniform samplerCube cubeMap;
uniform sampler2DShadow shadowMap;  //shadow map is just a texture
uniform sampler2D colorTexture;

out vec4 fragColor;       //final fragment color

//calculate diffuse component of lightning
float diffuseComponent(vec3 L, vec3 N){
	return clamp(dot(L, N), 0.0, 1.0);
} 

//calculate specular component of lightning
float specularComponent(vec3 L, vec3 N, vec3 H){
	if(dot(N, L) > 0){
		return pow(clamp(dot(N, H), 0.0, 1.0), 32.0);
	}
	return 0.0;
}

void main(void) {

	//  calculate reflection 
/*	vec4 incidentRay	= vec4(WS_camPos.x - WS_pos.x, WS_camPos.y - WS_pos.y, WS_camPos.z - WS_pos.z, WS_camPos.w - WS_pos.w);
	incidentRay			= normalize(incidentRay);
	vec3 incidentRay3	= vec3(incidentRay.x, incidentRay.y, incidentRay.z);
	vec3 reflectedRay	= reflect(incidentRay3, worldNormal);
*/
	//test
	vec4 negIncidentRay =vec4(WS_pos.x - WS_camPos.x, WS_pos.y - WS_camPos.y, WS_pos.z - WS_camPos.z, WS_pos.w - WS_camPos.w);
	negIncidentRay      = normalize(negIncidentRay);
	vec3 negIncidentRay3=vec3(negIncidentRay.x, negIncidentRay.y, negIncidentRay.z);
	
	//REFLECTION
//	vec3 reflectedRay	= reflect(negIncidentRay3, worldNormal);
    // lookup in reflection map
//	vec4 reflectedColor = texture(cubeMap, reflectedRay);  //textureCube = deprecated
	//test
	//vec4 reflectedColor = texture(cubeMap, worldNormal);

	//REFRACTION
	// relative index of refraction 
	//= is the ratio of the index of refraction in the medium containing the incident ray to that of the medium being entered
	float indexOfRefraction_Air  = 1.0003; 
	float indexOfRefractionGlass = 1.5;//
	float etaRatio = indexOfRefraction_Air / indexOfRefractionGlass;
	vec3 refractedRay   = refract(negIncidentRay3, worldNormal, etaRatio);
	// lookup in reflection map
	vec4 refractedColor = texture(cubeMap, refractedRay);
	

	//sample color from diffuse texture
	vec3 color = texture2D(colorTexture, tex_uv).rgb;

	// maybe TODO: Interpolation between reflected color and texture color, reflectivity (as interpolation factor) 
//	float reflectivity = 0.9;
//	reflectedColor.xyz = mix(color, reflectedColor.xyz, reflectivity);
	
	//calculate total intensity of lightning
	vec3 halfVector = normalize(directionToLight + directionToCamera);
	float ambientIntensity = 0.3;
	float diffuseIntensity = diffuseComponent(directionToLight, worldNormal);
	float specularIntensity = specularComponent(directionToLight, worldNormal, halfVector);

	//perform perspective division - homogenisierung
	vec3 tex_coords = Shadow_tex_coord.xyz/Shadow_tex_coord.w;

	//lookup depth value from shadow map
	float inShadow = texture(shadowMap, tex_coords.xyz - vec3(0,0,0.004));

	float totalIntensity = ambientIntensity + 0.7 * (diffuseIntensity * inShadow) + (specularIntensity * inShadow);

	//use color from texture and reflection map
//	fragColor.xyz = reflectedColor.xyz * totalIntensity; 
	fragColor.xyz = refractedColor.xyz * totalIntensity;
	fragColor.w = 1.0;
}









