#version 330 core

layout(location = 0) in vec3 position;     //attribute passsed by the application
layout(location = 1) in vec3 normal;
layout(location = 2) in vec3 color;
layout(location = 3) in vec2 uv;

uniform mat4 modelMatrix;		 //model matrix
uniform mat4 VP;				 // = P_cam * V_cam 
uniform mat4 depthTranslScalVP;  // = Mt * Ms * P_light * V_light 
uniform vec3 cameraPosition;
uniform vec3 LightPos;

out vec4 WS_pos;
out vec4 WS_camPos;
out vec4 Shadow_tex_coord;		 
out vec3 worldNormal;
out vec3 materialColor;
out vec2 tex_uv;
out vec3 directionToLight;
out vec3 directionToCamera;

void main(void) {

	directionToLight = vec3(LightPos.x - position.x, LightPos.y - position.y, LightPos.z - position.z);
	directionToLight = normalize(directionToLight);
	directionToCamera = vec3(cameraPosition.x - position.x, cameraPosition.y - position.y, cameraPosition.z - position.z);
	directionToCamera = normalize(directionToCamera);

	//standard trandformation
	gl_Position = VP * modelMatrix * vec4(position, 1.0);  //Output position of the vertex, in clip space
	
	WS_pos	  = modelMatrix * vec4(position, 1.0);            //wird gebraucht fuer incident ray Berechnung !
	WS_camPos = /*modelMatrix **/ vec4(cameraPosition, 1.0);      //wird gebraucht fuer incident ray Berechnung !
	worldNormal = normalize((modelMatrix * vec4(normal, 0.0)).xyz); 
	materialColor = color; 
	tex_uv = uv;

	//shadow texture coords in projected light space  
	Shadow_tex_coord = depthTranslScalVP * modelMatrix * vec4(position, 1.0);   
}
