#version 330 core   

in vec2 tex_uv;
in vec3 positionWorldSpace;
in vec3 eyeDirectionCameraSpace;
in vec3 lightDirectionCameraSpace;
in vec3 lightDirectionTangentSpace;
in vec3 eyeDirectionTangentSpace;

out vec4 fragColor;

uniform sampler2D colorTexture;
uniform sampler2D normalMap;
uniform mat3 MV3x3;

void main(void) {
	// Light emission properties
	vec3 lightColor = vec3(1,1,1);

	// Material properties
	vec3 diffuseColor = texture2D(colorTexture, tex_uv).rgb * 0.8;
	vec3 ambientColor = vec3(0.3,0.3,0.3) * diffuseColor;
	vec3 specularColor= vec3(0.4,0.4,0.4) * lightColor;
	
	// Get normal from texture in tangent space
	vec3 textureNormalTangentSpace = normalize(texture2D( normalMap, tex_uv ).rgb * 2.0 - 1.0);
	
	// Calculate cosTheta in tangent space
	vec3 n = textureNormalTangentSpace;
	vec3 l = normalize(lightDirectionTangentSpace);
	float cosTheta = clamp( dot(l,n), 0, 1 );

	// Calculate cosAlpha in tangent space
	vec3 E = normalize(eyeDirectionTangentSpace);
	vec3 H = normalize(E+l);
	float cosAlpha = pow(clamp( dot(n,H), 0, 1 ), 32);

	fragColor = vec4(
				ambientColor +
				diffuseColor * lightColor * cosTheta +
				specularColor * lightColor * cosAlpha, 1);
}
