#version 330 core   

layout(location = 0) in vec3 position;
layout(location = 1) in vec3 normal;
layout(location = 2) in vec3 color;
layout(location = 3) in vec2 uv;
layout(location = 4) in vec3 tangent;
layout(location = 5) in vec3 bitangent;

out vec2 tex_uv;
out vec3 positionWorldSpace;	// Position in World Space
out vec3 eyeDirectionCameraSpace;
out vec3 lightDirectionCameraSpace;
out vec3 lightDirectionTangentSpace;
out vec3 eyeDirectionTangentSpace;
out vec4 Shadow_tex_coord;

uniform mat3 MV3x3;				// Model-View Matrix, dropping translation
uniform mat4 P;					// Model-View-Projection Matrix
uniform mat4 V;					// View-Matrix
uniform	mat4 modelMatrix;	
uniform vec3 LightPos;

uniform mat4 depthTranslScalVP;  // = Mt * Ms * P_light * V_light 

void main(void) {
	// Output position of the vertex in clip space
	gl_Position = P * V * modelMatrix * vec4(position,1);

	// Output position of the vertex in world space
	positionWorldSpace = (modelMatrix * vec4(position,1)).xyz;

	// Output vector from vertex to eye in camera space
	vec3 vertexInCameraSpace = (V * modelMatrix * vec4(position,1)).xyz;
	eyeDirectionCameraSpace = vec3(0,0,0) - vertexInCameraSpace;

	// Output vector from vertex to the light in camera space
	vec3 lightPositionCameraSpace = (V * vec4(LightPos,1)).xyz;
	lightDirectionCameraSpace = lightPositionCameraSpace - vertexInCameraSpace; 

	// Building TBN (Tangent-Bitangent-Normal) Matrix to transform things from Camera Space to Tangent Space
	vec3 vertexNormalCameraspace    = MV3x3 * normalize(normal);
	vec3 vertexTangentCameraspace   = MV3x3 * normalize(tangent);
	vec3 vertexBitangentCameraspace = MV3x3 * normalize(bitangent);
	
	mat3 TBN = transpose(mat3(
		vertexTangentCameraspace,
		vertexBitangentCameraspace,
		vertexNormalCameraspace	));	

	// Transform light direction and eye direction to tangent space
	lightDirectionTangentSpace = TBN * lightDirectionCameraSpace;
	eyeDirectionTangentSpace   = TBN * eyeDirectionCameraSpace;

	tex_uv = uv;
} 
