#version 330 core   

in vec4 Shadow_tex_coord;          //passed on from VS
in vec3 worldNormal;
in vec3 materialColor;
in vec2 tex_uv;
in vec3 directionToLight;
in vec3 directionToCamera;
//in vec3 N;
//in vec3 L;

uniform sampler2DShadow shadowMap;  //shadow map is just a texture
uniform sampler2D colorTexture;

out vec4 fragColor;       //final fragment color

//calculate diffuse component of lightning
float diffuseComponent(vec3 L, vec3 N){
	return clamp(dot(L, N), 0.0, 1.0);
} 

//calculate specular component of lightning
float specularComponent(vec3 L, vec3 N, vec3 H){
	if(dot(N, L) > 0){
		return pow(clamp(dot(N, H), 0.0, 1.0), 32.0);
	}
	return 0.0;
}

void main(void) {

	//sample color from diffuse texture
	vec3 color = texture2D(colorTexture, tex_uv).rgb;
	//calculate total intensity of lightning
	vec3 halfVector = normalize(directionToLight + directionToCamera);
	float ambientIntensity = 0.3;
	float diffuseIntensity = diffuseComponent(directionToLight, worldNormal);
	float specularIntensity = specularComponent(directionToLight, worldNormal, halfVector);

	//perform perspective division - homogenisierung
	vec3 tex_coords = Shadow_tex_coord.xyz/Shadow_tex_coord.w;

	//read depth value from shadow map
	float inShadow = texture(shadowMap, tex_coords.xyz - vec3(0,0,0.004));
//	inShadow = max(min(inShadow, 1.0),0.0);

	float totalIntensity = ambientIntensity + 0.7 * (diffuseIntensity * inShadow) + (specularIntensity * inShadow);

	//use color from texture
	fragColor.xyz = color * totalIntensity; 
	fragColor.w = 1.0;
}
