#version 330 core  

in vec4 Shadow_tex_coord;          //passed on from VS
in vec3 worldNormal;
in vec3 materialColor;
in vec3 directionToLight;
in vec3 directionToCamera;

uniform sampler2DShadow shadowMap;  //shadow map is just a texture
uniform float numShades;

out vec4 fragColor;       //final fragment color

//calculate diffuse component of lightning
float diffuseComponent(vec3 L, vec3 N){
	return clamp(dot(L, N), 0.0, 1.0);
} 

//calculate specular component of lightning
float specularComponent(vec3 L, vec3 N, vec3 H){
	if(dot(N, L) > 0){
		return pow(clamp(dot(H, N), 0.0, 1.0), 80.0/*64.0*/);
	}
	return 0.0;
}

void main(void) {
	//sample color from diffuse texture
	vec3 color = materialColor;
	//calculate total intensity of lightning
	vec3 halfVector = normalize(directionToLight + directionToCamera);
	float ambientIntensity = 0.2;
	float diffuseIntensity = diffuseComponent(directionToLight, worldNormal);
	float specularIntensity = specularComponent(directionToLight, worldNormal, halfVector);

	//perform perspective division - homogenisierung
	vec3 tex_coords = Shadow_tex_coord.xyz/Shadow_tex_coord.w;
	//read depth value from shadow map
	float inShadow = texture(shadowMap, tex_coords.xyz - vec3(0,0,0.001));
//	inShadow = max(min(inShadow, 1.0),0.0);

	float totalIntensity = ambientIntensity + (diffuseIntensity * inShadow) + specularIntensity;

	//quantize intensity for cel shading
	float shadeIntensity = ceil(totalIntensity * numShades) / numShades;

	//use color from texture
	fragColor.xyz = color * shadeIntensity; 
	fragColor.w = 1.0;
}
